// MLP101CUSBHIDIO.cpp : code file
// MPULAB

#pragma once
#include "stdafx.h"
#include "MLP101CUSBHIDIO.h"

// NOTE - The following lib files must be added to the project 
// in Linker->Input->additional_dependencies and phyiscally to the project
// setupapi.lib hid.lib
// Also the following header file must be added to the project
// hidsdi.h hidpi.h hidusage.h


// Required functions for HID API
extern "C" {		// note this exten "C" is required 
#include "hidsdi.h"
#include "setupapi.h"
}


// CUSBHIDIO Constuctor
CUSBHIDIO::CUSBHIDIO()
{
	hDevice = 0;

}



// ------------------------------------------------------ //
// Closes the device
// ------------------------------------------------------ //
unsigned int CUSBHIDIO::CloseDevice(void)
{
	if(hDevice) {
		CloseHandle(hDevice);
		hDevice = 0;
		}
	return 0;
}	

// ------------------------------------------------------ //
// Find and open the USB HID IO Device
// ------------------------------------------------------ //
unsigned int CUSBHIDIO::OpenDevice(void)
{
	return(ScanForHIDDevice(USB_VID, USB_PID));
}




// ------------------------------------------------------ //
// ScanForHidDevice(VID,PID) - Scan HID device matching VID, PID
// 0 = Success
// 1 = No matching HID devices
// ------------------------------------------------------ //
unsigned int CUSBHIDIO::ScanForHIDDevice(unsigned int VID, unsigned int PID)
{
	//Use a series of API calls to find a HID with a matching Vendor,Product
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;
	GUID						HidGuid;
	HANDLE						hDevInfo;
	ULONG						Required;
	HIDD_ATTRIBUTES				Attributes;
	SP_DEVICE_INTERFACE_DATA	devInfoData;
	bool						LastDevice = FALSE;
	int							MemberIndex = 0;
	bool						MyDeviceDetected = FALSE; 
	LONG						Result;
	ULONG						Length;
	
	// Variable init	
	Length = 0;
	detailData = NULL;
	hDevice=NULL;
	MemberIndex = 0;
	LastDevice = FALSE;

	
	HidD_GetHidGuid(&HidGuid);	
	
	hDevInfo=SetupDiGetClassDevs(&HidGuid, NULL, NULL, DIGCF_PRESENT|DIGCF_INTERFACEDEVICE);		
	devInfoData.cbSize = sizeof(devInfoData);

	do	{
		MyDeviceDetected=FALSE;
		
		Result=SetupDiEnumDeviceInterfaces(hDevInfo, 0, &HidGuid, MemberIndex, &devInfoData);
		if (Result != 0)
			{	//A device has been detected, so get more information about it.

			Result = SetupDiGetDeviceInterfaceDetail(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);

			detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(Length);
			
			detailData -> cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			Result = SetupDiGetDeviceInterfaceDetail(hDevInfo, &devInfoData, detailData, Length, &Required, NULL);

			hDevice=CreateFile(detailData->DevicePath, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE,
								(LPSECURITY_ATTRIBUTES)NULL, OPEN_EXISTING, 0, NULL);

			Attributes.Size = sizeof(Attributes);
			Result = HidD_GetAttributes(hDevice, &Attributes);

			MyDeviceDetected = FALSE;
      if ((Attributes.VendorID == VID) && (Attributes.ProductID == PID))
      {
        MyDeviceDetected = TRUE;   // premark it found
        // To enable overlappedtransfer use: GetDeviceCapabilities() & PrepareForOverlappedTransfer()
        strncpy((char*)DeviceName, detailData->DevicePath, 512);		// save the devicename
      }
			else	{	//The PID and/or VID doesn't match. Close the device try the next one
				CloseHandle(hDevice); 
				hDevice = 0;
				} 

			//Free the memory used by the detailData structure (no longer needed).
			free(detailData);
		}  //if (Result != 0)

		else	{				// End of List - No HID devices detected!
			LastDevice=TRUE;	//SetupDiEnumDeviceInterfaces returned 0, so there are no more devices to check.
			}

		//If we haven't found the device yet, and haven't tried every available device,
		//try the next one.
		MemberIndex++;

	} // loop till either end of deivce list or we find our device 
	while ((LastDevice == FALSE) && (MyDeviceDetected == FALSE));


	SetupDiDestroyDeviceInfoList(hDevInfo);		//Free the memory reserved for hDevInfo by SetupDiClassDevs.

	if (MyDeviceDetected == FALSE) {
		// Device not found
		hDevice = 0;
		return(1);
		}
	else	{
		// Device Found
		HidD_SetNumInputBuffers(hDevice,1);					//sets the maximum number of input reports that the HID class driver ring buffer can hold for a specified top-level collection.
		return(0); // Success
		}
	
}

// ---------------------------------------------------------------- //
// Return ture if the device is currently openned
// ---------------------------------------------------------------- //
int CUSBHIDIO::IsOpen(void)
{	
	if(hDevice) return(1);
	else		return(0);
}


// ---------------------------------------------------------------- //
// Writes a command to the USB HID Device
// ---------------------------------------------------------------- //
int CUSBHIDIO::WriteCmd(unsigned char *pbuffer, unsigned int Length)
{
	if(!hDevice) return(1);

  wpacket[0] = 2;
  memcpy(&wpacket[1], pbuffer, Length);

	if(HidD_SetFeature(hDevice, wpacket, Length + 1)) {

		return(0);		// success
		}

	return(1);
}


// ---------------------------------------------------------------- //
// Reads device info
// Retruns zero on success, else non-zero
// Return data in a 16byte data buffer. Buffer must be predeclared by called
// ---------------------------------------------------------------- //
int CUSBHIDIO::GetDeviceInfo(pMPULABDeviceInfoStruct pInfo)
{
  rpacket[0] = 1;

  if (!HidD_GetFeature(hDevice, &rpacket, 65))
  {
    return(1);	// command failed
  }

	// now get the dat if the variable has been passed
	if(!pInfo) return(1); 
	memcpy(pInfo, &rpacket, sizeof(MPULABDeviceInfoStruct));
	return 0;
}


// ---------------------------------------------------------------- //
// Reads the I/O port pin values,
// BYTE0 = PORT0, BYTE1 = PORT1
// Retruns zero on success, else non-zero 
// ---------------------------------------------------------------- //
int CUSBHIDIO::ReadPorts(unsigned char *pbuffer)
{
  rpacket[0] = 3;

	if(!HidD_GetFeature(hDevice, &rpacket, 65))
  {
    return(1);	// command failed
  }
  memcpy(pbuffer, &rpacket, 64);
	
	return 0;
}


